/*****************************************************************************
 *
 * phasex.h
 *
 * PHASEX:  [P]hase [H]armonic [A]dvanced [S]ynthesis [EX]periment
 *
 * Copyright (C) 1999-2007 William Weston <weston@sysex.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *****************************************************************************/
#ifndef _PHASEX_H_
#define _PHASEX_H_
#include <pthread.h>


/* type to use for all floating point math */
/* changing this to double will impact performance, and is probably overkill */
typedef float float_t;


/*****************************************************************************
 *
 * Tunable Compile Time Constants
 *
 *****************************************************************************/

/* Default time for gui to sleep in idle function (in microseconds) */
#define DEFAULT_GUI_IDLE_SLEEP	50000

/* Default realtime thread priorities					*/
/* These can be changed at runtime in the preferences			*/
#define MIDI_THREAD_PRIORITY	77
#define ENGINE_THREAD_PRIORITY	82

/* Base tuning frequency, in Hz						*/
/* Use the -t command line flag to override at runtime			*/
#define A4FREQ			440.0

/* Number of micro-tuning steps between halfsteps			*/
/* A value of 120 seems to provide good harmonics			*/
#define TUNING_RESOLUTION	120
#define F_TUNING_RESOLUTION	120.0		/* must equal above */

/* Factor by which the filter is oversampled.				*/
/* Harmonics are highly dependent on the prime factors in this value.	*/
/* Increase for richer harmonics and more stability at high resonance.	*/
/* Decrease to save CPU cycles or for thinner harmonics.		*/
/* A value of 6 seems to provide good harmonics at reasonable cost.	*/
#define FILTER_OVERSAMPLE	6
#define F_FILTER_OVERSAMPLE	6.0		/* must equal above */

/* Number of samples for a single wave period in the osc wave table	*/
/* Larger values sound cleaner, but burn more memory			*/
/* Smaller values sound grittier, but burn less memory			*/
/* On some CPUs, performance degrades with larger lookup tables		*/
/* Values with more small prime factors provide better harmonics	*/
/*   than large powers of 2.  Try these values first:			*/
/* 24576 28800 32400 44100 48000 49152 57600 64800 75600 73728 86400	*/
/* 88200 96000 97200 98304 115200 129600 132300 144000 162000 176400	*/
#define WAVEFORM_SIZE		57600
#define F_WAVEFORM_SIZE		57600.0		/* must equal above */

/* Size of the lookup table for logarithmic envelope curves.		*/
/* envelope curve table size shouldn't drop much below 14400		*/
#define ENV_CURVE_SIZE		28800
#define F_ENV_CURVE_SIZE        28800.0		/* must equal above */

/* Total polyphony */
/* As a general rule of thumb, this should be somewhat lower than	*/
/*   one voice per 100 MHz CPU power.  YMMV.				*/
#define NUM_VOICES		8

/* User files and directories for configs, patches, midimaps, etc.	*/
/* Since these are not full paths, any new code should reference the	*/
/*   globals provided at the bottom of this file instead.		*/
#define USER_DATA_DIR		".phasex"
#define USER_PATCH_DIR		"user-patches"
#define USER_MIDIMAP_DIR	"user-midimaps"
#define USER_BANK_FILE		"patchbank"
#define USER_PATCHDUMP_FILE	"patchdump"
#define USER_CONFIG_FILE	"phasex.cfg"
#define SYS_DEFAULT_PATCH	"default.phx"

/* define this if there are problems when disconnecting from JACK */
//define JACK_DEACTIVATE_BEFORE_CLOSE

/* Misc */
//define EXTRA_DEBUGGING	1
//define DEBUG_SAMPLE_VALUES	1



/*****************************************************************************
 *
 * Non-Tunable Constants
 *
 *****************************************************************************/

/* Realtime thread scheduling policies (only SCHED_FIFO has been tested) */
#define MIDI_THREAD_POLICY	SCHED_FIFO
#define ENGINE_THREAD_POLICY	SCHED_FIFO

/* Number of per-voice oscs and per-part lfos */
#define NUM_OSCS	       	4
#define NUM_LFOS	       	4
#define LFO_OFF			(NUM_LFOS)
#define MOD_OFF			(NUM_OSCS)

/* three options for sample rate: 1, 1/2, or 2 */
#define SAMPLE_RATE_NORMAL	0
#define SAMPLE_RATE_UNDERSAMPLE	1
#define SAMPLE_RATE_OVERSAMPLE	2

/* Update after adding new waveforms */
#define NUM_WAVEFORMS		12

/* Maximum delay time, in samples */
#define DELAY_MAX		720000
#define CHORUS_MAX		1024

/* Even-multiple octaves work best. */
/* Must be able to handle patch transpose + part transpose + pitchbend + fm */
#define FREQ_SHIFT_HALFSTEPS	384
#define F_FREQ_SHIFT_HALFSTEPS	384.0
#define FREQ_SHIFT_TABLE_SIZE	(FREQ_SHIFT_HALFSTEPS * TUNING_RESOLUTION)
#define F_FREQ_SHIFT_TABLE_SIZE	(F_FREQ_SHIFT_HALFSTEPS * F_TUNING_RESOLUTION)
#define FREQ_SHIFT_ZERO_OFFSET	(F_FREQ_SHIFT_TABLE_SIZE / 2.0)

/* max number of samples to use in the ringbuffer */
#define PHASEX_MAX_BUFSIZE	2048

/* Compile fixups */
#if !defined(PATH_MAX)
# define PATH_MAX		1024
#endif



/* globals from phasex.c */
extern int debug;
extern int shutdown;

extern pthread_t jack_thread_p;
extern pthread_t midi_thread_p;
extern pthread_t engine_thread_p;
extern pthread_t gtkui_thread_p;


extern char user_data_dir[PATH_MAX];
extern char user_patch_dir[PATH_MAX];
extern char user_midimap_dir[PATH_MAX];
extern char user_bank_file[PATH_MAX];
extern char user_patchdump_file[PATH_MAX];
extern char user_config_file[PATH_MAX];
extern char sys_default_patch[PATH_MAX];


/* functions from phasex.c */
void phasex_shutdown(const char *msg);


#endif /* _PHASEX_H_ */
